@@ -101,10 +101,6 @@ class CoverageTest(
         self.last_command_output = None
         self.last_module_name = None
 
-    def xfail(self, msg):
-        """Mark this test as an expected failure."""
-        pytest.xfail(msg)
-
     def clean_local_file_imports(self):
         """Clean up the results of calls to `import_local_file`.
 
@@ -352,6 +348,10 @@ class CoverageTest(
         ret_actual = command_line(args)
         self.assertEqual(ret_actual, ret)
 
+    # Some distros rename the coverage command, and need a way to indicate
+    # their new command name to the tests. This is here for them to override,
+    # for example:
+    # https://salsa.debian.org/debian/pkg-python-coverage/-/blob/master/debian/patches/02.rename-public-programs.patch
     coverage_command = "coverage"
 
     def run_command(self, cmd):
@@ -501,3 +501,8 @@ def command_line(args):
     script = CoverageScript()
     ret = script.command_line(shlex.split(args))
     return ret
+
+
+def xfail(condition, reason):
+    """A decorator to mark as test as expected to fail."""
+    return pytest.mark.xfail(condition, reason=reason, strict=True)
