@@ -1,8 +1,9 @@
 # Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
-# For details: https://bitbucket.org/ned/coveragepy/src/default/NOTICE.txt
+# For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt
 
 """Oddball cases for testing coverage.py"""
 
+import os.path
 import sys
 
 from flaky import flaky
@@ -93,7 +94,7 @@ class RecursionTest(CoverageTest):
                 )
 
     def test_long_recursion_recovery(self):
-        # Test the core of bug 93: https://bitbucket.org/ned/coveragepy/issue/93
+        # Test the core of bug 93: https://github.com/nedbat/coveragepy/issues/93
         # When recovering from a stack overflow, the Python trace function is
         # disabled, but the C trace function is not.  So if we're using a
         # Python trace function, we won't trace anything after the stack
@@ -118,14 +119,14 @@ class RecursionTest(CoverageTest):
         cov = coverage.Coverage()
         self.start_import_stop(cov, "recur")
 
-        pytrace = (cov.collector.tracer_name() == "PyTracer")
+        pytrace = (cov._collector.tracer_name() == "PyTracer")
         expected_missing = [3]
         if pytrace:                                 # pragma: no metacov
             expected_missing += [9, 10, 11]
 
         _, statements, missing, _ = cov.analysis("recur.py")
         self.assertEqual(statements, [1, 2, 3, 5, 7, 8, 9, 10, 11])
-        self.assertEqual(missing, expected_missing)
+        self.assertEqual(expected_missing, missing)
 
         # Get a warning about the stackoverflow effect on the tracing function.
         if pytrace:                                 # pragma: no metacov
@@ -140,7 +141,7 @@ class MemoryLeakTest(CoverageTest):
     """Attempt the impossible: test that memory doesn't leak.
 
     Note: this test is truly unusual, and has had a colorful history.  See
-    for example: https://bitbucket.org/ned/coveragepy/issue/186
+    for example: https://github.com/nedbat/coveragepy/issues/186
 
     It may still fail occasionally, especially on PyPy.
 
@@ -275,7 +276,7 @@ class PyexpatTest(CoverageTest):
         self.assertEqual(missing, [])
 
         # Make sure pyexpat isn't recorded as a source file.
-        # https://bitbucket.org/ned/coveragepy/issues/419/nosource-no-source-for-code-path-to-c
+        # https://github.com/nedbat/coveragepy/issues/419
         files = cov.get_data().measured_files()
         self.assertFalse(
             any(f.endswith("pyexpat.c") for f in files),
@@ -398,15 +399,6 @@ class ExceptionTest(CoverageTest):
 class DoctestTest(CoverageTest):
     """Tests invoked with doctest should measure properly."""
 
-    def setUp(self):
-        super(DoctestTest, self).setUp()
-
-        # This test case exists because Python 2.4's doctest module didn't play
-        # well with coverage. Nose fixes the problem by monkeypatching doctest.
-        # I want to be sure there's no monkeypatch and that I'm getting the
-        # doctest module that users of coverage will get.
-        assert 'doctest' not in sys.modules
-
     def test_doctest(self):
         self.check_coverage('''\
             def return_arg_or_void(arg):
@@ -433,7 +425,7 @@ class DoctestTest(CoverageTest):
 class GettraceTest(CoverageTest):
     """Tests that we work properly with `sys.gettrace()`."""
     def test_round_trip_in_untraced_function(self):
-        # https://bitbucket.org/ned/coveragepy/issues/575/running-doctest-prevents-complete-coverage
+        # https://github.com/nedbat/coveragepy/issues/575
         self.make_file("main.py", """import sample""")
         self.make_file("sample.py", """\
             from swap import swap_it
@@ -466,7 +458,7 @@ class GettraceTest(CoverageTest):
         self.assertEqual(missing, [])
 
     def test_setting_new_trace_function(self):
-        # https://bitbucket.org/ned/coveragepy/issues/436/disabled-coverage-ctracer-may-rise-from
+        # https://github.com/nedbat/coveragepy/issues/436
         self.check_coverage('''\
             import sys
 
@@ -543,14 +535,14 @@ class GettraceTest(CoverageTest):
 class ExecTest(CoverageTest):
     """Tests of exec."""
     def test_correct_filename(self):
-        # https://bitbucket.org/ned/coveragepy/issues/380/code-executed-by-exec-excluded-from
+        # https://github.com/nedbat/coveragepy/issues/380
         # Bug was that exec'd files would have their lines attributed to the
         # calling file.  Make two files, both with ~30 lines, but no lines in
         # common.  Line 30 in to_exec.py was recorded as line 30 in main.py,
         # but now it's fixed. :)
         self.make_file("to_exec.py", """\
             \n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n
-            print("var is {0}".format(var))         # line 31
+            print("var is {}".format(var))         # line 31
             """)
         self.make_file("main.py", """\
             namespace = {'var': 17}
@@ -571,11 +563,23 @@ class ExecTest(CoverageTest):
         self.assertEqual(statements, [31])
         self.assertEqual(missing, [])
 
+    def test_unencodable_filename(self):
+        # https://github.com/nedbat/coveragepy/issues/891
+        if env.PYVERSION < (3, 0):
+            self.skipTest("Python 2 can't seem to compile the file.")
+        self.make_file("bug891.py", r"""exec(compile("pass", "\udcff.py", "exec"))""")
+        cov = coverage.Coverage()
+        self.start_import_stop(cov, "bug891")
+        # Saving would fail trying to encode \udcff.py
+        cov.save()
+        files = [os.path.basename(f) for f in cov.get_data().measured_files()]
+        assert "bug891.py" in files
+
 
 class MockingProtectionTest(CoverageTest):
     """Tests about protecting ourselves from aggressive mocking.
 
-    https://bitbucket.org/ned/coveragepy/issues/416/coverage-40-is-causing-existing-unit-tests
+    https://github.com/nedbat/coveragepy/issues/416
 
     """
     def test_os_path_exists(self):
